//
// This file is part of the Marabou extension for PlaJA (2019 - 2024).
//

#include <unordered_map>
#include "enum_option_values.h"
#include "../assertions.h"

namespace PLAJA_OPTION {

    std::unique_ptr<EnumOptionValue> str_to_enum_option_value(const std::string& str) {
        static const std::unordered_map<std::string, EnumOptionValue> str_to_config {
            { "none", EnumOptionValue::None },
#ifdef BUILD_TO_NUXMV
            { "bmc", EnumOptionValue::Bmc },
            { "sbmc", EnumOptionValue::SBmc },
            { "ipa-bmc", EnumOptionValue::IpaBmc },
            { "ipa-cegar", EnumOptionValue::IpaCegar },
            { "cegar", EnumOptionValue::Cegar },
            { "ic3", EnumOptionValue::Ic3 },
            { "coi", EnumOptionValue::Coi },
#endif
        };

        const auto it = str_to_config.find(str);
        return it == str_to_config.cend() ? nullptr : std::make_unique<EnumOptionValue>(it->second);
    }

    std::string enum_option_value_to_str(EnumOptionValue value) {
        switch (value) {
            case EnumOptionValue::None: { return "none"; }
#ifdef BUILD_TO_NUXMV
            case EnumOptionValue::Bmc: { return "bmc"; }
            case EnumOptionValue::SBmc: { return "sbmc"; }
            case EnumOptionValue::IpaBmc: { return "ipa-bmc"; }
            case EnumOptionValue::IpaCegar: { return "ipa-cegar"; }
            case EnumOptionValue::Cegar: { return "cegar"; }
            case EnumOptionValue::Ic3: { return "ic3"; }
            case EnumOptionValue::Coi: { return "coi"; }
#endif
            default: PLAJA_ABORT
        }
    }

}