//
// This file is part of the PlaJA code base (2019 - 2019).
//

#ifndef PLAJA_CONSTANT_DECLARATION_H
#define PLAJA_CONSTANT_DECLARATION_H

#include <memory>
#include "ast_element.h"
#include "commentable.h"

// forward declaration:
class Expression;
class DeclarationType;

class ConstantDeclaration: public AstElement, public Commentable {
private:
    std::string name;
    // int id; // unique among all constants, larger equal zero smaller than number of constants (or -1)
    std::unique_ptr<DeclarationType> declarationType; // basic type or bounded type
    std::unique_ptr<Expression> value;
public:
    ConstantDeclaration();
    ~ConstantDeclaration() override;

    // setter:
    inline void set_name(const std::string& name_) {name = name_;}
    // inline void set_id(int id_n) {id = id_n;}
    void set_declarationType(std::unique_ptr<DeclarationType>&& decl_type);
    void set_value(std::unique_ptr<Expression>&& val);

    // getter:
    inline const std::string& get_name() const {return name;}
    // inline int get_id() const {return id;}
    inline DeclarationType* get_type() {return declarationType.get();}
    inline const DeclarationType* get_type() const {return declarationType.get();}
    inline Expression* get_value() {return value.get();}
    inline const Expression* get_value() const {return value.get();}

    /**
     * Deep copy of a variable declaration.
     * @return
     */
    std::unique_ptr<ConstantDeclaration> deepCopy() const;

    // override:
    void accept(AstVisitor* astVisitor) override;
	void accept(AstVisitorConst* astVisitor) const override;
};



#endif //PLAJA_CONSTANT_DECLARATION_H
