//
// This file is part of the PlaJA code base (2019 - 2019).
//

#ifndef PLAJA_ARRAY_ACCESS_EXPRESSION_H
#define PLAJA_ARRAY_ACCESS_EXPRESSION_H

#include <memory>
#include "variable_expression.h"

class ArrayAccessExpression final: public LValueExpression {
private:
    std::unique_ptr<VariableExpression> accessedArray; // currently restricted, in general arbitrary expression of array type
    std::unique_ptr<Expression> index;

    /* Auxiliary for runtime checks. */
    void check_offset(PLAJA::integer offset) const;

public:
    ArrayAccessExpression();
    ~ArrayAccessExpression() override;
    DELETE_CONSTRUCTOR(ArrayAccessExpression)

    /* setter */

    inline void set_accessedArray(std::unique_ptr<VariableExpression>&& accessed_array) { accessedArray = std::move(accessed_array); }

    inline void set_index(std::unique_ptr<Expression>&& index_) { index = std::move(index_); }

    /* special aux */

    void set_accessedArray(const VariableDeclaration& var_decl);

    void set_index(PLAJA::integer index_);

    /* getter */

    [[nodiscard]] inline Expression* get_accessedArray() { return accessedArray.get(); }

    [[nodiscard]] inline const Expression* get_accessedArray() const { return accessedArray.get(); }

    [[nodiscard]] inline Expression* get_index() { return index.get(); }

    [[nodiscard]] inline const Expression* get_index() const { return index.get(); }

    /* override */
    void assign(StateBase& target, const StateBase* source, const Expression* value) const override;
    PLAJA::integer access_integer(const StateBase& target, const StateBase* source) const override;
    PLAJA::floating access_floating(const StateBase& target, const StateBase* source) const override;
    [[nodiscard]] VariableID_type get_variable_id() const override;
    [[nodiscard]] const Expression* get_array_index() const override;
    [[nodiscard]] VariableIndex_type get_variable_index(const StateBase* state) const override;
    using LValueExpression::get_variable_index;
    PLAJA::integer evaluateInteger(const StateBase* state) const override;
    PLAJA::floating evaluateFloating(const StateBase* state) const override;
    [[nodiscard]] bool is_constant() const override;
    bool equals(const Expression* exp) const override;
    [[nodiscard]] std::size_t hash() const override;
    const DeclarationType* determine_type() override;
    void accept(AstVisitor* ast_visitor) override;
    void accept(AstVisitorConst* ast_visitor) const override;
    //std::unique_ptr<Expression> deepCopy_Exp() const override;
    [[nodiscard]] std::unique_ptr<LValueExpression> deep_copy_lval_exp() const override;
    [[nodiscard]] std::unique_ptr<ArrayAccessExpression> deepCopy() const;

};

#endif //PLAJA_ARRAY_ACCESS_EXPRESSION_H
