//
// This file is part of the PlaJA code base (2019 - 2019).
//

#include "constant_expression.h"
#include "../../visitor/ast_visitor.h"
#include "../../visitor/ast_visitor_const.h"
#include "../type/declaration_type.h"
#include "constant_value_expression.h"

ConstantExpression::ConstantExpression(std::string name_, const DeclarationType* decl_type):
    name(std::move(name_)) {
    resultType = decl_type->deep_copy_decl_type();
}

ConstantExpression::~ConstantExpression() = default;

// setter:

void ConstantExpression::set_value(std::unique_ptr<ConstantValueExpression>&& val) { value = std::move(val); }

// override:

PLAJA::integer ConstantExpression::evaluateInteger(const StateBase* state) const {
    JANI_ASSERT(value)
    return value->evaluateInteger(state);
}

PLAJA::floating ConstantExpression::evaluateFloating(const StateBase* state) const {
    JANI_ASSERT(value)
    return value->evaluateFloating(state);
}

bool ConstantExpression::is_constant() const { return true; }

bool ConstantExpression::equals(const Expression* exp) const {
    auto* other = dynamic_cast<const ConstantExpression*>(exp);
    if (not other) { return false; }

    /* set value of the constant is not part of its identity
    if (this->value) {
        if (other->value) {
            if (not this->value->equals(other->value.get())) { return false; }
        }
        else { return false; }
    } else {
        if (other->value) { return false; }
    }
    */

    return this->name == other->name;

}

std::size_t ConstantExpression::hash() const {
    constexpr unsigned prime = 31;
    std::size_t result = 1;
    result = prime * result + PLAJA_UTILS::hashString(name);
    // if (value) result = prime * result + value->hash(); // set value of the constant is not part of its identity
    return result;
}

const DeclarationType* ConstantExpression::determine_type() {
    PLAJA_ASSERT(resultType) // expressions may not return nullptr
    return resultType.get();
}

void ConstantExpression::accept(AstVisitor* ast_visitor) { ast_visitor->visit(this); }

void ConstantExpression::accept(AstVisitorConst* ast_visitor) const { ast_visitor->visit(this); }

std::unique_ptr<Expression> ConstantExpression::deepCopy_Exp() const {
    std::unique_ptr<ConstantExpression> copy(new ConstantExpression(name, resultType.get()));
    if (value) { copy->set_value(value->deepCopy_ConstantValueExp()); }
    return copy;
}

