//
// This file is part of the PlaJA code base (2019 - 2019).
//

#ifndef PLAJA_CONSTANT_EXPRESSION_H
#define PLAJA_CONSTANT_EXPRESSION_H

#include <memory>
#include "expression.h"

// forward declaration:
class DeclarationType;
class ConstantValueExpression;

/**
 * Reference to a constant declaration.
 */
class ConstantExpression: public Expression {
private:
    std::string name;
    std::unique_ptr<ConstantValueExpression> value; // must be of type of referenced constant
public:
    explicit ConstantExpression(std::string name_, const DeclarationType* decl_type);
    ~ConstantExpression() override;

    // setter:
    void set_value(std::unique_ptr<ConstantValueExpression>&& val);

    // getter:
    [[nodiscard]] inline const std::string& get_name() const { return name; }
    inline ConstantValueExpression* get_value() { return value.get(); }
    [[nodiscard]] inline const ConstantValueExpression* get_value() const { return value.get(); }

    // override:
    PLAJA::integer evaluateInteger(const StateBase* state) const override;
    PLAJA::floating evaluateFloating(const StateBase* state) const override;
    [[nodiscard]] bool is_constant() const override;
    bool equals(const Expression* exp) const override;
    [[nodiscard]] std::size_t hash() const override;
    const DeclarationType* determine_type() override;
    void accept(AstVisitor* astVisitor) override;
	void accept(AstVisitorConst* astVisitor) const override;
    [[nodiscard]] std::unique_ptr<Expression> deepCopy_Exp() const override;
};


#endif //PLAJA_CONSTANT_EXPRESSION_H
