//
// This file is part of the PlaJA code base (2019 - 2019).
//

#include "derivative_expression.h"
#include "../../../exception/not_implemented_exception.h"
#include "../../visitor/ast_visitor.h"
#include "../../visitor/ast_visitor_const.h"
#include "../type/real_type.h"

DerivativeExpression::DerivativeExpression(std::string name_, VariableID_type var_id, const DeclarationType* declType, VariableIndex_type var_index):
    name(std::move(name_))
    , id(var_id)
    , index(var_index) {
    JANI_ASSERT(declType)
    resultType = declType->deep_copy_decl_type();
}

DerivativeExpression::~DerivativeExpression() = default;

// override:

bool DerivativeExpression::is_constant() const { return false; }

bool DerivativeExpression::is_proposition() const { throw NotImplementedException(__PRETTY_FUNCTION__); }

const DeclarationType* DerivativeExpression::determine_type() {
    if (resultType) { return resultType.get(); }
    else {
        resultType = std::make_unique<RealType>();
        return resultType.get();
    }
}

void DerivativeExpression::accept(AstVisitor* ast_visitor) { ast_visitor->visit(this); }

void DerivativeExpression::accept(AstVisitorConst* ast_visitor) const { ast_visitor->visit(this); }

std::unique_ptr<Expression> DerivativeExpression::deepCopy_Exp() const { return std::make_unique<DerivativeExpression>(name, id, resultType.get(), index); }
