//
// This file is part of the PlaJA code base (2019 - 2019).
//

#ifndef PLAJA_DERIVATIVE_EXPRESSION_H
#define PLAJA_DERIVATIVE_EXPRESSION_H

#include "expression.h"

// forward declaration:
class DeclarationType;

class DerivativeExpression: public Expression {
private:
    std::string name;
    VariableID_type id;
    VariableIndex_type index; // state index of underlying variable
public:
    DerivativeExpression(std::string name_, VariableID_type var_id, const DeclarationType* declType, VariableIndex_type var_index = static_cast<VariableIndex_type>(-1));
    ~DerivativeExpression() override;

    // setter:
    inline void set_id(VariableID_type var_id) {id = var_id;}
    inline void set_index(VariableIndex_type var_index) {index = var_index;}

    // getter:
    [[nodiscard]] inline const std::string& get_name() const {return name;}
    [[nodiscard]] inline VariableID_type get_id() const {return id;}
    [[nodiscard]] inline VariableIndex_type get_index() const {return index;}

    // override:
    [[nodiscard]] bool is_constant() const override;
    [[nodiscard]] bool is_proposition() const override;
    const DeclarationType* determine_type() override;
    void accept(AstVisitor* astVisitor) override;
	void accept(AstVisitorConst* astVisitor) const override;
    [[nodiscard]] std::unique_ptr<Expression> deepCopy_Exp() const override;
};


#endif //PLAJA_DERIVATIVE_EXPRESSION_H
