//
// This file is part of the PlaJA code base (2019 - 2019).
//

#include "variable_expression.h"
#include "../../../search/states/state_base.h"
#include "../../visitor/ast_visitor.h"
#include "../../visitor/ast_visitor_const.h"
#include "../type/declaration_type.h"
#include "../variable_declaration.h"

VariableExpression::VariableExpression(const VariableDeclaration& var_decl):
    name(var_decl.get_name())
    , id(var_decl.get_id())
    , index(var_decl.get_index()) {
    resultType = var_decl.get_type()->deep_copy_decl_type();
}

VariableExpression::VariableExpression(std::string name, VariableID_type var_id, const DeclarationType* variable_type, VariableIndex_type var_index):
    name(std::move(name))
    , id(var_id)
    , index(var_index) {
    resultType = variable_type->deep_copy_decl_type();
}

VariableExpression::~VariableExpression() = default;

/* override */

void VariableExpression::assign(StateBase& target, const StateBase* source, const Expression* value) const {
    PLAJA_ASSERT(/*0 <= index && */ index < target.size())
    if (resultType->is_floating_type()) { target.assign_float<true>(index, value->evaluateFloating(source)); }
    else {
        PLAJA_ASSERT(resultType->is_integer_type() or resultType->is_boolean_type())
        target.assign_int<true>(index, value->evaluateInteger(source));
    }
    
    PLAJA_ASSERT(target.is_valid())
}

PLAJA::integer VariableExpression::access_integer(const StateBase& target, const StateBase* /*source*/) const { return target.get_int(index); }

PLAJA::floating VariableExpression::access_floating(const StateBase& target, const StateBase* /*source*/) const { return target.get_float(index); }

VariableID_type VariableExpression::get_variable_id() const { return id; }

VariableIndex_type VariableExpression::get_variable_index(const StateBase* /*state*/) const { return index; }

PLAJA::integer VariableExpression::evaluateInteger(const StateBase* state) const { return state->get_int(index); }

PLAJA::floating VariableExpression::evaluateFloating(const StateBase* state) const { return state->operator[](index); }

bool VariableExpression::is_constant() const { return false; }

bool VariableExpression::equals(const Expression* exp) const {
    auto* other = dynamic_cast<const VariableExpression*>(exp);
    if (not other) { return false; }
    return this->id == other->id;
}

std::size_t VariableExpression::hash() const { return id; }

const DeclarationType* VariableExpression::determine_type() {
    PLAJA_ASSERT(resultType)
    return resultType.get(); // expressions may not return nullptr
}

void VariableExpression::accept(AstVisitor* ast_visitor) { ast_visitor->visit(this); }

void VariableExpression::accept(AstVisitorConst* ast_visitor) const { ast_visitor->visit(this); }

std::unique_ptr<LValueExpression> VariableExpression::deep_copy_lval_exp() const { return deep_copy(); }

//

std::unique_ptr<VariableExpression> VariableExpression::deep_copy() const { return std::make_unique<VariableExpression>(name, id, resultType.get(), index); }








