//
// This file is part of the PlaJA code base (2019 - 2019).
//

#ifndef PLAJA_VARIABLE_EXPRESSION_H
#define PLAJA_VARIABLE_EXPRESSION_H

#include <memory>
#include "expression.h"
#include "lvalue_expression.h"

// forward declaration:
class VariableDeclaration;

/**
 * Reference to a variable (declaration).
 */
class VariableExpression final: public LValueExpression {

private:
    std::string name;
    VariableID_type id;
    VariableIndex_type index; // state index of underlying variable

public:
    explicit VariableExpression(const VariableDeclaration& var_decl);
    explicit VariableExpression(std::string name, VariableID_type var_id, const DeclarationType* variable_type, VariableIndex_type var_index = static_cast<VariableIndex_type>(-1));
    ~VariableExpression() override;
    DELETE_CONSTRUCTOR(VariableExpression)

    /* setter */
    inline void set_name(const std::string& name_) { name = name_; }

    inline void set_id(VariableID_type var_id) { id = var_id; }

    inline void set_index(VariableIndex_type var_index) { index = var_index; }

    /* getter */
    [[nodiscard]] inline const std::string& get_name() const { return name; }

    [[nodiscard]] inline VariableID_type get_id() const { return id; }

    [[nodiscard]] inline VariableIndex_type get_index() const { return index; }

    /* override */
    void assign(StateBase& target, const StateBase* source, const Expression* value) const override; // as LValue
    PLAJA::integer access_integer(const StateBase& target, const StateBase* source) const override;
    PLAJA::floating access_floating(const StateBase& target, const StateBase* source) const override;
    [[nodiscard]] VariableID_type get_variable_id() const override;
    [[nodiscard]] VariableIndex_type get_variable_index(const StateBase* state) const override;
    using LValueExpression::get_variable_index;
    PLAJA::integer evaluateInteger(const StateBase* state) const override;
    PLAJA::floating evaluateFloating(const StateBase* state) const override;
    [[nodiscard]] bool is_constant() const override;
    bool equals(const Expression* exp) const override;
    [[nodiscard]] std::size_t hash() const override;
    const DeclarationType* determine_type() override;
    void accept(AstVisitor* ast_visitor) override;
    void accept(AstVisitorConst* ast_visitor) const override;
    [[nodiscard]] std::unique_ptr<LValueExpression> deep_copy_lval_exp() const override;
    //
    [[nodiscard]] std::unique_ptr<VariableExpression> deep_copy() const;

};

#endif //PLAJA_VARIABLE_EXPRESSION_H
