//
// This file is part of the PlaJA code base (2019 - 2019).
//

#include "array_type.h"
#include "../../../utils/utils.h"
#include "../../visitor/ast_visitor.h"
#include "../../visitor/ast_visitor_const.h"

ArrayType::ArrayType() = default;

ArrayType::~ArrayType() = default;

// override:

void ArrayType::accept(AstVisitor* astVisitor) { astVisitor->visit(this); }

void ArrayType::accept(AstVisitorConst* astVisitor) const { astVisitor->visit(this); }

bool ArrayType::is_array_type() const { return true; }

bool ArrayType::is_boolean() const { return elementType->is_boolean(); }

bool ArrayType::is_integer() const { return elementType->is_integer(); }

bool ArrayType::is_floating() const { return elementType->is_floating(); }

DeclarationType::TypeKind ArrayType::get_kind() const { return TypeKind::Array; }

bool ArrayType::is_assignable_from(const DeclarationType& assigned_type) const {
    if (TypeKind::Array == assigned_type.get_kind()) {
        JANI_ASSERT(elementType and dynamic_cast<const ArrayType*>(&assigned_type)->get_element_type())
        return elementType->is_assignable_from(*PLAJA_UTILS::cast_ref<ArrayType>(assigned_type).get_element_type());
    } else { return false; }
}

/*
bool ArrayType::operator==(const DeclarationType& ref_type) const {
    if (TypeKind::ARRAY == ref_type.get_kind()) {
        JANI_ASSERT(elementType && (dynamic_cast<const ArrayType&>(ref_type)).elementType)
        return (*elementType) == (*(static_cast<const ArrayType&>(ref_type)).elementType);
    } else { return false; }
}
*/

std::unique_ptr<DeclarationType> ArrayType::infer_common(const DeclarationType& ref_type) const {
    if (TypeKind::Array == ref_type.get_kind()) {

        JANI_ASSERT(elementType and dynamic_cast<const ArrayType*>(&ref_type)->get_element_type())

        std::unique_ptr<DeclarationType> common_element_type = elementType->infer_common(*PLAJA_UTILS::cast_ref<ArrayType>(ref_type).get_element_type());

        if (common_element_type) { // not nullptr
            std::unique_ptr<ArrayType> inferred_type(new ArrayType());
            inferred_type->set_element_type(std::move(common_element_type));
            return inferred_type;
        } else { return nullptr; }

    } else { return nullptr; }
}

std::unique_ptr<DeclarationType> ArrayType::deep_copy_decl_type() const {
    std::unique_ptr<ArrayType> copy(new ArrayType());
    if (elementType) { copy->set_element_type(elementType->deep_copy_decl_type()); }
    return copy;
}
