//
// This file is part of the Marabou extension for PlaJA (2019 - 2023).
//

#include "split_on_locs.h"
#include "../../ast/expression/non_standard/location_value_expression.h"
#include "../../ast/expression/non_standard/state_condition_expression.h"
#include "../../ast/expression/binary_op_expression.h"
#include "split_on_binary_op.h"

std::unique_ptr<StateConditionExpression> ExtractStateCondition::extract(std::list<std::unique_ptr<Expression>>& conjuncts) {

    ExtractStateCondition extract_state_condition;

    for (auto it = conjuncts.cbegin(); it != conjuncts.end();) {

        extract_state_condition.moved = false;
        (*it)->accept(&extract_state_condition);
        if (extract_state_condition.moved) { it = conjuncts.erase(it); }
        else { ++it; }

    }

    return std::move(extract_state_condition.stateCondition);

}

ExtractStateCondition::ExtractStateCondition():
    stateCondition(nullptr)
    , moved(false) {

}

ExtractStateCondition::~ExtractStateCondition() = default;

void ExtractStateCondition::visit(StateConditionExpression* exp) {

    moved = true;

    if (not stateCondition) { stateCondition = exp->move(); }
    else {

        /* Merge locations. */
        stateCondition->reserve(stateCondition->get_size_loc_values() + exp->get_size_loc_values());
        for (auto it = exp->init_loc_value_iterator(); !it.end(); ++it) { stateCondition->add_loc_value(it.set()); }

        /**/
        if (exp->get_constraint()) {

            if (stateCondition->get_constraint()) {

                auto merge_expr = std::make_unique<BinaryOpExpression>(BinaryOpExpression::AND);
                merge_expr->set_left(stateCondition->set_constraint());
                merge_expr->set_right(exp->set_constraint());
                merge_expr->determine_type();
                stateCondition->set_constraint(std::move(merge_expr));

            } else { stateCondition->set_constraint(exp->set_constraint()); }

        }

    }

}

/**********************************************************************************************************************/

SplitOnLocs::SplitOnLocs() = default;

SplitOnLocs::~SplitOnLocs() = default;

std::unique_ptr<StateConditionExpression> SplitOnLocs::split(std::unique_ptr<Expression>&& expr) {

    auto splits = SplitOnBinaryOp::split(std::move(expr), BinaryOpExpression::AND);

    auto state_condition = ExtractStateCondition::extract(splits);

    if (state_condition) { if (state_condition->get_constraint()) { splits.push_back(state_condition->set_constraint()); } }
    else { state_condition = std::make_unique<StateConditionExpression>(); }

    state_condition->set_constraint(SplitOnBinaryOp::to_expr(std::move(splits), BinaryOpExpression::AND));

    return state_condition;

}