//
// This file is part of the PlaJA code base (2019 - 2019).
//

#ifndef PLANNING_IN_VARIABLE_ID_VISITOR_H
#define PLANNING_IN_VARIABLE_ID_VISITOR_H

#include <unordered_map>
#include <vector>
#include "ast_visitor.h"
#include "../using_parser.h"

class ModelInformation;

class SetVarIndexBase: public AstVisitor {
protected:
    SetVarIndexBase();
public:
    ~SetVarIndexBase() override = 0;
    DELETE_CONSTRUCTOR(SetVarIndexBase)

    void visit(Destination* destination) override;
    void visit(Edge* edge) override;
    void visit(LocationValueExpression* exp) override;
};

class SetVarIndexByID: public SetVarIndexBase {
private:
    const std::vector<VariableIndex_type>* idToIndex;
    const Model* model;
    const ModelInformation* modelInfo; // if present check bounds variable-value expressions

    explicit SetVarIndexByID(const std::vector<VariableIndex_type>& id_to_index, const ModelInformation* model_info);
    explicit SetVarIndexByID(const Model& model);
public:
    ~SetVarIndexByID() override;
    DELETE_CONSTRUCTOR(SetVarIndexByID)

    void visit(Destination* destination) override;
    void visit(Edge* edge) override;
    void visit(LocationValueExpression* exp) override;
    void visit(DerivativeExpression* exp) override;
    void visit(VariableExpression* exp) override;
    void visit(VariableValueExpression* exp) override;
    static void set_vars(const std::vector<VariableIndex_type>& id_to_index, AstElement* ast_el, const ModelInformation* model_info = nullptr);
    static void set_vars(const Model& model, AstElement& ast_el);
};

class [[maybe_unused]] SetVarIndexByIndex: public SetVarIndexBase {
private:
    const std::vector<int>& indexToIndex; // mapping from old index to new

    explicit SetVarIndexByIndex(const std::vector<int>& index_to_index);
public:
    ~SetVarIndexByIndex() override;
    DELETE_CONSTRUCTOR(SetVarIndexByIndex)

    void visit(Destination* destination) override;
    void visit(Edge* edge) override;
    void visit(LocationValueExpression* exp) override;
    void visit(DerivativeExpression* exp) override;
    void visit(VariableExpression* exp) override;
    static void set_vars(const std::vector<int>& index_to_index, AstElement* ast_el);
};

#endif //PLANNING_IN_VARIABLE_ID_VISITOR_H
