//
// This file is part of the PlaJA code base (2019 - 2020).
//

#ifndef PLAJA_NEURAL_NETWORK_H
#define PLAJA_NEURAL_NETWORK_H

#include <torch/nn/module.h>
#include "../using_policy_learning.h"

namespace TORCH_IN_PLAJA {

    class NeuralNetwork: public torch::nn::Module {

    private:
        std::vector<unsigned int> layerSizes;
        unsigned int inputSize;
        unsigned int outputSize;
        std::vector<torch::nn::Linear> forwardingFuncs;

        /* For to nnet. */
        std::vector<TORCH_IN_PLAJA::Floating> lowerBounds;
        std::vector<TORCH_IN_PLAJA::Floating> upperBounds;

    public:
        explicit NeuralNetwork(std::vector<unsigned int> layers, std::vector<TORCH_IN_PLAJA::Floating> lower_bounds = std::vector<TORCH_IN_PLAJA::Floating>(), std::vector<TORCH_IN_PLAJA::Floating> upper_bounds = std::vector<TORCH_IN_PLAJA::Floating>());
        ~NeuralNetwork() override;

        torch::Tensor forward(torch::Tensor x); // Manual does parameter not by copy.

        void load_from_nnet(const std::string& file_name);

        void to_nnet(const std::string& file_name);

        void dump() const;

    };

    TORCH_MODULE_IMPL(NeuralNetworkModule, NeuralNetwork);

}

#endif //PLAJA_NEURAL_NETWORK_H
