//
// This file is part of the PlaJA code base (2021).
//

#ifndef PLAJA_BMC_H
#define PLAJA_BMC_H

#include "../fd_adaptions/search_engine.h"

class BoundedMC: public SearchEngine {
public:
    enum BMCMode {
        UNSAFE, // check for unsafe paths
        LOOP_FREE, // check for loop free paths
        UNSAFE_LOOP_FREE,
        LOOP_FREE_UNSAFE
    };

protected:
    const BMCMode bmcMode;
    // options:
    const unsigned int maxSteps;
    const bool findShortestPathOnly;
    const bool constrainNoStart;
    const bool constrainLoopFree;
    const bool constrainNoReach;

private:
    int steps;
    int shortestUnsafePath;
    int longestLoopFreePath;
    std::unique_ptr<std::string> bmcStatsFile;

    SearchStatus initialize() override;
    SearchStatus step() override;

    SearchStatus check_loop_free();
    SearchStatus check_unsafe();

    [[nodiscard]] virtual bool check_start_smt() const = 0;
    [[nodiscard]] virtual bool check_loop_free_smt(unsigned int steps_) const = 0;
    [[nodiscard]] virtual bool check_unsafe_smt(unsigned int steps_) const = 0;

public:
    explicit BoundedMC(const PLAJA::Configuration& config);
    ~BoundedMC() override;
    DELETE_CONSTRUCTOR(BoundedMC)

};

namespace BOUNDED_MC { extern std::string print_supported_bmc_modes(); }

#endif //PLAJA_BMC_H
