//
// This file is part of the PlaJA code base (2019 - 2021).
//

#ifndef PLAJA_INVARIANT_STRENGTHENING_H
#define PLAJA_INVARIANT_STRENGTHENING_H


#include "../fd_adaptions/search_engine.h"
#include "../smt/using_smt.h"


// forward declaration
class ModelMarabou;
class ModelZ3;
class SuccessorGeneratorAbstract;
class SearchStatisticsPA;
class State;
//
namespace MARABOU_IN_PLAJA { class MarabouQuery; }
namespace PLAJA { class SMTSolverM; }


class InvariantStrengthening final: public SearchEngine {

private:
    std::unique_ptr<z3::expr> invariant; // i.e. of the form "!unsafety constraint AND  ..."
    std::unique_ptr<z3::expr> non_invariant; // i.e. of the form "unsafety constraint OR ..."
    std::unique_ptr<SearchStatisticsPA> searchStatistics;
    //
    std::unique_ptr<ModelZ3> modelZ3;
    std::unique_ptr<SuccessorGeneratorAbstract> successorGenerator;
    std::unique_ptr<PLAJA::SMTSolverM> solver;
    // NN
    std::unique_ptr<ModelMarabou> modelMarabou;
    std::unique_ptr<MARABOU_IN_PLAJA::MarabouQuery> query;
    std::unique_ptr<z3::expr_vector> forwardingStructure;
    std::vector<z3::expr> outputInterfacePerAction;
    // flags
    bool perActionQuery;

    SearchStatus initialize() override;
    SearchStatus finalize() override;
    SearchStatus step() override;

    // auxiliaries:
    bool check_incremental();
    [[nodiscard]] z3::expr extract_expansion_in_z3() const; // with respect to explore

    // override stats:
    SearchStatistics& get_search_statistics() override;
    void print_statistics() const override;
    void stats_to_csv(std::ofstream& file) const override;
    void stat_names_to_csv(std::ofstream& file) const override;

public:
    explicit InvariantStrengthening(const PLAJA::Configuration& config);
    ~InvariantStrengthening() override;

};


#endif //PLAJA_INVARIANT_STRENGTHENING_H
