//
// This file is part of the PlaJA code base (2019 - 2021).
//

#ifndef PLAJA_INITIALIZE_BASE_H
#define PLAJA_INITIALIZE_BASE_H

struct InitializeBase {

    template<SearchEngineNonProbBase::SearchEngineType engine_type, typename SearchEngine_t, typename SearchNode_t>
    inline static SearchEngine::SearchStatus initialize(SearchEngineNonProbBase* const engine, SearchEngine_t* const derived_engine, StateID_type initial_id) { // i.e., the "this" pointer

        const auto& initial_state = engine->searchSpace->get_state(initial_id);
        engine->searchStatistics->inc_attr_unsigned(PLAJA::StatsUnsigned::GENERATED_STATES);
        engine->searchStatistics->inc_attr_unsigned(PLAJA::StatsUnsigned::START_STATES);

        auto initial_node = derived_engine->searchSpace->add_node(initial_id);
        initial_node.set_reached_init();
        engine->searchStatistics->inc_attr_unsigned(PLAJA::StatsUnsigned::REACHABLE_STATES);
        if constexpr (engine_type != SearchEngineNonProbBase::BFS) { derived_engine->stateSpace->add_state_information(initial_id); }

        // goal check
        if (engine->goal and engine->goal->evaluateInteger(&initial_state)) {
            PLAJA_LOG("Initial state is goal!")
            engine->searchStatistics->inc_attr_unsigned(PLAJA::StatsUnsigned::GOAL_STATES);
            engine->searchSpace->add_goal_path_frontier(initial_id);
            initial_node().set_goal();
            if constexpr (engine_type == SearchEngineNonProbBase::BFS) { return SearchEngine::SOLVED; }
        }

        /* Add to frontier. */
        engine->frontier.push_back(initial_id);

        return SearchEngine::IN_PROGRESS;

    }

};

#endif //PLAJA_INITIALIZE_BASE_H
