//
// This file is part of the PlaJA code base (2019 - 2019).
//

#include "breadth_first_search.h"
#include "../../option_parser/option_parser.h"
#include "../../option_parser/plaja_options.h"
#include "../../parser/ast/expression/expression.h"
#include "../../parser/ast/model.h"
#include "../../stats/stats_base.h"
#include "../fd_adaptions/state.h"
#include "../successor_generation/successor_generator.h"
#include "policy/policy_restriction.h"
#include "search_space_bfs.h"

BFSearch::BFSearch(const PLAJA::Configuration& config):
    SearchEngineNonProbBase(config)
    , searchSpace(new SearchSpaceBFS(model->get_model_information())) {

    searchStatistics->add_int_stats(PLAJA::StatsInt::PATH_LENGTH, "PathLength", -1); // Set stats properly.

    SearchEngineBase::set_search_space(searchSpace.get());
}

BFSearch::~BFSearch() = default;

//

#include "base/initialize_base.h"

SearchEngine::SearchStatus BFSearch::initialize(StateID_type initial_id) {
    return InitializeBase::initialize<SearchEngineType::BFS, BFSearch, SearchNodeInfoBase>(this, this, initial_id);
}

#include "base/step_base.h"

SearchEngine::SearchStatus BFSearch::step() {
    return StepBase::step<SearchEngineType::BFS, BFSearch, SearchNodeInfoBase>(this, this);
}

SearchEngine::SearchStatus BFSearch::finalize() {

    if (not searchSpace->goal_path_frontier_empty()) {

        PLAJA_LOG("Goal state reachable!")

        if (PLAJA_GLOBAL::has_value_option(PLAJA_OPTION::savePath)) {
            const auto path_len = PLAJA_UTILS::cast_numeric<int>(searchSpace->dump_path(searchSpace->goal_frontier_state(), model).size());
            searchStatistics->set_attr_int(PLAJA::StatsInt::PATH_LENGTH, path_len); // Set stats properly.
        } else {
            const auto path_len = PLAJA_UTILS::cast_numeric<int>(searchSpace->trace_path(searchSpace->goal_frontier_state()).size());
            searchStatistics->set_attr_int(PLAJA::StatsInt::PATH_LENGTH, path_len);
        }

    } else {
        searchStatistics->set_attr_int(PLAJA::StatsInt::PATH_LENGTH, 0);
    }

    return SearchEngine::FINISHED;
}