//
// This file is part of the PlaJA code base (2019 - 2021).
//

#include <memory>
#include "policy_restriction.h"
#include "../../factories/configuration.h"
#include "../../fd_adaptions/state.h"
#include "../../information/jani2nnet/jani_2_nnet.h"
#include "../../information/property_information.h"
#include "nn_policy.h"

PolicyRestriction::PolicyRestriction(const PLAJA::Configuration& config, const Jani2NNet& policy_interface):
    nnPolicy(&policy_interface.load_policy(config))
    , cachedState(STATE::none) {}

PolicyRestriction::~PolicyRestriction() = default;

//

const Jani2NNet& PolicyRestriction::get_nn_interface() const { return nnPolicy->get_nn_interface(); }

//

std::unique_ptr<PolicyRestriction> PolicyRestriction::construct_policy_restriction(const PLAJA::Configuration& config) {
    PLAJA_ASSERT(config.has_sharable_const(PLAJA::SharableKey::PROP_INFO))
    auto prop_info = config.get_sharable_const<PropertyInformation>(PLAJA::SharableKey::PROP_INFO);
    if (prop_info->get_nn_interface()) {
        PLAJA_LOG("Constructing NN-based policy restriction ...")
        return std::make_unique<PolicyRestriction>(config, *prop_info->get_nn_interface());
    } else { return nullptr; }
}

//

bool PolicyRestriction::is_enabled(const State& state, ActionLabel_type action_label) const {
    const StateID_type queried_state_id = state.get_id_value();
    if (queried_state_id != cachedState) {
        cachedState = queried_state_id;
        nnPolicy->evaluate(state);
    }
    return nnPolicy->is_chosen(action_label, PLAJA_NN::argMaxPrecision);
}

bool PolicyRestriction::is_enabled(const StateBase& state, ActionLabel_type action_label) const { return nnPolicy->is_chosen(state, action_label, PLAJA_NN::argMaxPrecision); }

PLAJA::floating PolicyRestriction::get_selection_delta() const { return nnPolicy->get_selection_delta(); }

bool PolicyRestriction::is_chosen_with_tolerance() const { return nnPolicy->is_chosen_with_tolerance(); }

FCT_IF_DEBUG(void PolicyRestriction::dump_cache() const { nnPolicy->dump_cached_outputs(); })