//
// This file is part of the PlaJA code base (2019 - 2021).
//

#ifndef PLAJA_POLICY_RESTRICTION_H
#define PLAJA_POLICY_RESTRICTION_H

#include <memory>
#include "../../../assertions.h"
#include "../../factories/forward_factories.h"
#include "../../information/jani2nnet/forward_jani_2_nnet.h"
#include "../../information/jani2nnet/using_jani2nnet.h"
#include "../../states/forward_states.h"
#include "../../using_search.h"
#include "forward_policy.h"

class PolicyRestriction {

private:
    const NNPolicy* nnPolicy;
    mutable StateID_type cachedState;

public:
    explicit PolicyRestriction(const PLAJA::Configuration& config, const Jani2NNet& policy_interface);
    ~PolicyRestriction();
    DELETE_CONSTRUCTOR(PolicyRestriction)

    [[nodiscard]] const Jani2NNet& get_nn_interface() const;

    static std::unique_ptr<PolicyRestriction> construct_policy_restriction(const PLAJA::Configuration& config);

    [[nodiscard]] bool is_enabled(const State& state, ActionLabel_type action_label) const;
    [[nodiscard]] bool is_enabled(const StateBase& state, ActionLabel_type action_label) const;

    [[nodiscard]] inline bool is_pruned(const State& state, ActionLabel_type action_label) const { return !is_enabled(state, action_label); }

    [[nodiscard]] inline bool is_pruned(const StateBase& state, ActionLabel_type action_label) const { return !is_enabled(state, action_label); }

    [[nodiscard]] PLAJA::floating get_selection_delta() const;
    [[nodiscard]] bool is_chosen_with_tolerance() const;

    FCT_IF_DEBUG(void dump_cache() const;)

};

#endif //PLAJA_POLICY_RESTRICTION_H
