//
// This file is part of the PlaJA code base (2019 - 2020).
//

#include "space_explorer.h"
#include "../../option_parser/plaja_options.h"
#include "../../parser/ast/model.h"
#include "../../parser/ast/expression/expression.h"
#include "../../stats/stats_base.h"
#include "../factories/non_prob_search/non_prob_search_options.h"
#include "../factories/configuration.h"
#include "../fd_adaptions/state.h"
#include "../successor_generation/successor_generator.h"
#include "../prob_search/state_space.h"
#include "base/search_space_base.h"
#include "base/search_node_info_base.h"
#include "policy/policy_restriction.h"

/**********************************************************************************************************************/

namespace SPACE_EXPLORER {

    struct SearchNodeInfo final: public SearchNodeInfoBase {};

    class SearchSpace final: public SearchSpace_<SearchNodeInfo> {
    public:
        explicit SearchSpace(const ModelInformation& model_info):
            SearchSpace_<SearchNodeInfo>(model_info) {}

        ~SearchSpace() override = default;
        DELETE_CONSTRUCTOR(SearchSpace)
    };

}

/**********************************************************************************************************************/

SpaceExplorer::SpaceExplorer(const PLAJA::Configuration& config):
    SearchEngineExplorerBase(config)
    , searchSpace(new SPACE_EXPLORER::SearchSpace(model->get_model_information()))
    , computeGoalPaths(config.get_bool_option(PLAJA_OPTION::computeGoalPaths)) {

    SearchEngineBase::set_search_space(searchSpace.get());
}

SpaceExplorer::~SpaceExplorer() = default;

#include "base/initialize_base.h"

SearchEngine::SearchStatus SpaceExplorer::initialize(StateID_type initial_id) {
    return InitializeBase::initialize<SearchEngineType::EXPLORER, SpaceExplorer, SPACE_EXPLORER::SearchNodeInfo>(this, this, initial_id);
}

#include "base/step_base.h"

SearchEngine::SearchStatus SpaceExplorer::step() { return StepBase::step<SearchEngineType::EXPLORER, SpaceExplorer, SPACE_EXPLORER::SearchNodeInfo>(this, this); }

SearchEngine::SearchStatus SpaceExplorer::finalize() {
    if (computeGoalPaths) { compute_goal_path_states(); }
    return FINISHED;
}

/* Override stats. */

void SpaceExplorer::print_statistics() const {
    SearchEngineExplorerBase::print_statistics();
    if (PLAJA_GLOBAL::has_value_option(PLAJA_OPTION::saveStateSpace)) { stateSpace->dump_state_space(); }
}
