//
// This file is part of the PlaJA code base (2019 - 2020).
//

#ifndef PLAJA_PA_CEGAR_H
#define PLAJA_PA_CEGAR_H

#include <memory>
#include <vector>
#include <unordered_set>
#include <list>
#include "../../../include/ct_config_const.h"
#include "../../../parser/ast/expression/forward_expression.h"
#include "../../../parser/ast/forward_ast.h"
#include "../../../utils/default_constructors.h"
#include "../../using_search.h"
#include "../../factories/predicate_abstraction/forward_factories_pa.h"
#include "../../fd_adaptions/search_engine.h"
#include "forward_cegar.h"

class SearchPAPath;

class PACegar final: public SearchEngine {
    friend class PACegarTest;

private:
    // cached:
    std::unique_ptr<Properties> cachedProperties;

    // abstract search
    std::unique_ptr<Property> paProperty;
    std::unique_ptr<PredicatesStructure> predicates;
    std::unique_ptr<PropertyInformation> propInfoSub;
    unsigned int maxNumPreds;
    // sub-engine (have to cache to prohibit dangling pointer in case of signal handling)
    std::unique_ptr<SearchPAPath> cegarSearch;

    // concrete path certification:
    std::unique_ptr<SpuriousnessChecker> spuriousnessChecker;

    PAExpression* get_pa_expression();
    void cache_pa_property();
    void predicates_to_file() const;

    // override:
    SearchStatus initialize() override;
    SearchStatus step() override;

public:
    explicit PACegar(const SearchEngineConfigPA& config);
    ~PACegar() override;
    DELETE_CONSTRUCTOR(PACegar)

    // stats
    void update_statistics() const override;
    void print_statistics() const override;
    void stats_to_csv(std::ofstream& file) const override;
    void stat_names_to_csv(std::ofstream& file) const override;
};

#endif //PLAJA_PA_CEGAR_H
