//
// This file is part of the PlaJA code base (2019 - 2022).
//

#ifndef PLAJA_STATE_QUEUE_H
#define PLAJA_STATE_QUEUE_H

#include <list>
#include "../../using_search.h"
#include "../../../stats/stats_interface.h"

// forward declaration:
namespace SEARCH_SPACE_PA { struct SearchNodeView; }

/** Base class to order states based on heuristic values */
class StateQueue: public PLAJA::StatsInterface {

protected:

    // template<typename Queue_t>
    // Queue_t& cast_queue() { DYNAMIC_TYPE_IS(queue.get(), Queue_t) return static_cast<Queue_t&>(*queue); }

    StateQueue();
public:
    virtual ~StateQueue() = 0;

    virtual void increment() = 0;

    virtual void push(const SEARCH_SPACE_PA::SearchNodeView& state_node) = 0;

    // interface:
    virtual StateID_type pop() = 0;
    [[nodiscard]] virtual std::size_t size() const = 0;
    [[nodiscard]] virtual bool empty() const = 0;
    virtual void clear() = 0;
    //
    [[nodiscard]] virtual StateID_type top() const = 0;
    [[nodiscard]] virtual std::list<StateID_type> to_list() const = 0;
};

class FIFOStateQueue final: public StateQueue {
private:
    std::list<StateID_type> queue;
public:
    FIFOStateQueue();
    ~FIFOStateQueue() final;

    // interface:
    StateID_type pop() override;
    [[nodiscard]] std::size_t size() const override;
    [[nodiscard]] bool empty() const override;
    void clear() override;
    //
    [[nodiscard]] StateID_type top() const override;
    [[nodiscard]] std::list<StateID_type> to_list() const override;

    void increment() override { clear(); }

    void push(const SEARCH_SPACE_PA::SearchNodeView& state_node) final;
};


#endif //PLAJA_STATE_QUEUE_H
