//
// This file is part of the PlaJA code base (2019 - 2023).
//

#ifndef PLAJA_INC_STATE_SPACE_MT_H
#define PLAJA_INC_STATE_SPACE_MT_H

#include "../../../stats/forward_stats.h"
#include "incremental_state_space.h"
#include "pa_match_tree.h"
#include "source_node.h"

namespace SEARCH_SPACE_PA {

    struct MtNodeInformation final: public PaMtNodeInformation {

    private:
        SourceNode<const PAMatchTree::Node*> sourceNode;

    public:
        MtNodeInformation() = default;
        ~MtNodeInformation() override = default;
        DELETE_CONSTRUCTOR(MtNodeInformation)

        [[nodiscard]] const SourceNode<const PAMatchTree::Node*>& get_source_node() const { return sourceNode; }

        [[nodiscard]] SourceNode<const PAMatchTree::Node*>& get_source_node() { return sourceNode; }

    };

    class IncStateSpaceMt final: public IncStateSpace {

    private:
        std::shared_ptr<StateRegistryPa> stateRegistryPa;
        std::shared_ptr<PAMatchTree> matchTree;
        PLAJA::StatsBase* stats;

        [[nodiscard]] std::unique_ptr<StateID_type> transition_exists_internal(const AbstractState& source, UpdateOpID_type update_op_id, const AbstractState& successor) const;

    public:
        explicit IncStateSpaceMt(const PLAJA::Configuration& config, std::shared_ptr<StateRegistryPa> state_reg_pa);
        ~IncStateSpaceMt() final;
        DELETE_CONSTRUCTOR(IncStateSpaceMt)

        void update_stats() const override;

        void increment() override;

        void clear_coarser() override;

        /**************************************************************************************************************/

        void set_excluded_label(const AbstractState& src, ActionLabel_type action_label) override;

        void set_excluded_op(const AbstractState& src, ActionOpID_type action_op) override;

        void store_entailments(const AbstractState& src, UpdateOpID_type update_op_id, std::vector<std::pair<PredicateIndex_type, bool>>&& entailments_) override;

        void add_transition_witness(const AbstractState& src, UpdateOpID_type update_op_id, StateID_type src_witness, StateID_type suc_witness) override;

        void set_excluded(const AbstractState& src, UpdateOpID_type update_op_id, const AbstractState& excluded_successor) override;

        void refine_for(const AbstractState& source) override;

        [[nodiscard]] bool is_excluded_label(const AbstractState& source, ActionLabel_type action_label) const override;

        [[nodiscard]] bool is_excluded_op(const AbstractState& source, ActionOpID_type action_op) const override;

        [[nodiscard]] PredicatesNumber_type load_entailments(PredicateState& target, const AbstractState& source, UpdateOpID_type update_op, const PredicateRelations* predicate_relations) const override;

        [[nodiscard]] std::unique_ptr<StateID_type> transition_exists(const AbstractState& source, UpdateOpID_type update_op_id, const AbstractState& successor) const override;

        /**************************************************************************************************************/

        [[nodiscard]] PredicateTraverser init_predicate_traverser() const final {
#ifdef LAZY_PA
            return PredicateTraverser::init(*matchTree);
#else
            return PredicateTraverser::init(stateRegistryPa->get_predicates_size());
#endif
        }

        FCT_IF_LAZY_PA(FCT_IF_DEBUG([[nodiscard]] inline bool is_closed(const PaStateBase& pa_state, bool minimal) { return PredicateTraverser::is_closed(*matchTree, pa_state, minimal); }))

    };

}

#endif //PLAJA_INC_STATE_SPACE_MT_H
