//
// This file is part of the PlaJA code base (2019 - 2022).
//

#include "incremental_state_space.h"
#include "../../factories/predicate_abstraction/pa_options.h"
#include "../../factories/configuration.h"
#include "inc_state_space_reg.h"
#include "inc_state_space_mt.h"

SEARCH_SPACE_PA::IncStateSpace::IncStateSpace() = default;

#ifdef USE_REF_REG

SEARCH_SPACE_PA::IncStateSpace::IncStateSpace(const PLAJA::Configuration& config, std::shared_ptr<StateRegistryPA> state_reg_pa):
    incStateSpaceRef(nullptr) {

    if (config.get_bool_option(PLAJA_OPTION::incremental_search)) {
        // pointless to do use incStateSpaceRef in non-incremental mode
        // additionally this would require several modifications (also in search space class)
#ifdef USE_INC_REG
        incStateSpaceRef = std::make_unique<SEARCH_SPACE_PA::IncStateSpaceMt>(config, std::move(state_reg_pa));
#endif

#ifdef USE_MATCH_TREE
        incStateSpaceRef = std::make_unique<SEARCH_SPACE_PA::IncStateSpaceReg>(config, std::move(state_reg_pa));
#endif
    }

}

#endif

SEARCH_SPACE_PA::IncStateSpace::~IncStateSpace() = default;

/**********************************************************************************************************************/

#ifndef NDEBUG

void SEARCH_SPACE_PA::IncStateSpace::check_is_excluded_label(bool rlt, const AbstractState& source, ActionLabel_type action_label) const {
    PLAJA_ASSERT(not incStateSpaceRef or incStateSpaceRef->is_excluded_label(source, action_label) == rlt);
}

void SEARCH_SPACE_PA::IncStateSpace::check_is_excluded_op(bool rlt, const AbstractState& source, ActionOpID_type action_op) const {
    PLAJA_ASSERT(not incStateSpaceRef or incStateSpaceRef->is_excluded_op(source, action_op) == rlt)
}

void SEARCH_SPACE_PA::IncStateSpace::check_transition_exists(const std::unique_ptr<StateID_type>& rlt, const AbstractState& source, UpdateOpID_type update_op_id, const AbstractState& successor) const {

    if (incStateSpaceRef) {

        auto rlt_ref = incStateSpaceRef->transition_exists(source, update_op_id, successor);

        PLAJA_ASSERT((not rlt and not rlt_ref) or (rlt and rlt_ref))

        PLAJA_ASSERT((not rlt and not rlt_ref) or *rlt == *rlt_ref)

    }

}

#endif
