//
// This file is part of the PlaJA code base (2019 - 2022).
//

#ifndef PLAJA_INCREMENTAL_STATE_SPACE_H
#define PLAJA_INCREMENTAL_STATE_SPACE_H

#include <memory>
#include <vector>
#include "../../../include/ct_config_const.h"
#include "../../../assertions.h"
#include "../../factories/forward_factories.h"
#include "../optimization/forward_optimization_pa.h"
#include "../pa_states/forward_pa_states.h"
#include "../using_predicate_abstraction.h"
#include "forward_inc_state_space_pa.h"
#include "predicate_traverser.h"

namespace SEARCH_SPACE_PA {

    class IncStateSpace {

    protected:
        FIELD_IF_DEBUG(std::unique_ptr<IncStateSpace> incStateSpaceRef;)

        FCT_IF_DEBUG(void check_is_excluded_label(bool rlt, const AbstractState& source, ActionLabel_type action_label) const;)

        FCT_IF_DEBUG(void check_is_excluded_op(bool rlt, const AbstractState& source, ActionOpID_type action_op) const;)

        FCT_IF_DEBUG(void check_transition_exists(const std::unique_ptr<StateID_type>& rlt, const AbstractState& source, UpdateOpID_type update_op_id, const AbstractState& successor) const;)

        IncStateSpace();
#ifdef USE_REF_REG
        IncStateSpace(const PLAJA::Configuration& config, std::shared_ptr<StateRegistryPA> state_reg_pa); // used to construct ref state space
#endif

    public:
        virtual ~IncStateSpace() = 0;
        DELETE_CONSTRUCTOR(IncStateSpace)

        virtual void update_stats() const = 0;

        virtual void increment() = 0;

        virtual void clear_coarser() = 0;

        /**************************************************************************************************************/

        virtual void set_excluded_label(const AbstractState& src, ActionLabel_type action_label) = 0;

        virtual void set_excluded_op(const AbstractState& src, ActionOpID_type action_op) = 0;

        virtual void store_entailments(const AbstractState& src, UpdateOpID_type update_op_id, std::vector<std::pair<PredicateIndex_type, bool>>&& entailments) = 0;

        virtual void add_transition_witness(const AbstractState& src, UpdateOpID_type update_op_id, StateID_type src_witness, StateID_type suc_witness) = 0;

        virtual void set_excluded(const AbstractState& src, UpdateOpID_type update_op_id, const AbstractState& excluded_successor) = 0;

        virtual void refine_for(const AbstractState& source) = 0;

        [[nodiscard]] virtual bool is_excluded_label(const AbstractState& source, ActionLabel_type action_label) const = 0;

        [[nodiscard]] virtual bool is_excluded_op(const AbstractState& source, ActionOpID_type action_op) const = 0;

        [[nodiscard]] virtual PredicatesNumber_type load_entailments(PredicateState& target, const AbstractState& source, UpdateOpID_type update_op_id, const PredicateRelations* predicate_relations) const = 0;

        [[nodiscard]] virtual std::unique_ptr<StateID_type> transition_exists(const AbstractState& source, UpdateOpID_type update_op_id, const AbstractState& successor) const = 0;

        /**************************************************************************************************************/

        [[nodiscard]] virtual PredicateTraverser init_predicate_traverser() const = 0;

    };

}

#endif //PLAJA_INCREMENTAL_STATE_SPACE_H
