//
// This file is part of the PlaJA code base (2019 - 2023).
//

#ifndef PLAJA_PRED_ENTAILMENT_CACHE_INTERFACE_H
#define PLAJA_PRED_ENTAILMENT_CACHE_INTERFACE_H

#include <list>
#include <unordered_map>
#include <vector>
#include "../../../assertions.h"
#include "../using_predicate_abstraction.h"
#include "../search_space/forward_search_space.h"
#include "../forward_pa.h"

/*
 * Cached entailment info (for entailment per op-update) from previous iterations of PA computation (in coarser state spaces).
 * Entailment info is assumed to be dense,
 * i.e., numPredsChecked indicates the predicate source state size up to which entailment has been checked for each successor predicate.
 */
struct PredEntailmentCacheInterface final {
private:
    // to store new entailment information
    SearchSpacePABase* searchSpace;
    const PATransitionStructure* transition;
    const class PredicateRelations* predicateRelations;

    // existing entailment information
    std::list<const std::vector<std::pair<PredicateIndex_type, bool>>*> cache;
    PredicatesNumber_type numPredsChecked;

public:
    PredEntailmentCacheInterface(SearchSpacePABase* search_space, const PATransitionStructure* transition_, const class PredicateRelations* predicate_relations);
    ~PredEntailmentCacheInterface() = default;
    DELETE_CONSTRUCTOR(PredEntailmentCacheInterface)

    [[nodiscard]] inline const class PredicateRelations* _predicate_relations() const { return predicateRelations; }

    void load_entailment(class PredicateState& target) const;

    [[nodiscard]] inline PredicatesNumber_type _num_pred_checked() const { return numPredsChecked; }

    void store_entailment(const std::unordered_map<PredicateIndex_type, bool>& entailments) const;

};

#endif //PLAJA_PRED_ENTAILMENT_CACHE_INTERFACE_H
