//
// This file is part of the PlaJA code base (2019 - 2022).
//

#ifndef PLAJA_PA_TRANSITION_STRUCTURE_H
#define PLAJA_PA_TRANSITION_STRUCTURE_H

#include <memory>
#include "../using_search.h"

// forward declaration:
class AbstractState;
class ActionOpPA;
class UpdatePA;

struct PATransitionStructure final {

private:
    // basic data
    std::unique_ptr<AbstractState> source;
    std::unique_ptr<AbstractState> target;
    ActionLabel_type actionLabel;
    ActionOpID_type actionOpID;
    UpdateIndex_type updateIndex;
    // more fancy data:
    const ActionOpPA* actionOp;
    const UpdatePA* update;
    // even more fancy:

public:
    PATransitionStructure();
    PATransitionStructure(const PATransitionStructure& other);
    ~PATransitionStructure();

    PATransitionStructure& operator=(const PATransitionStructure& other);

    void set_source(std::unique_ptr<AbstractState>&& pa_src);
    void unset_source();
    void set_target(std::unique_ptr<AbstractState>&& pa_target);
    void unset_target();

    inline void set_action_label(ActionLabel_type action_label) { actionLabel = action_label; }
    inline void unset_action_label() { actionLabel = ACTION::noneLabel; }
    //
    inline void set_action_op(ActionOpID_type action_op_id) { actionOpID = action_op_id; }
    void set_action_op(const ActionOpPA& action_op);
    inline void unset_action_op() { actionOpID = ACTION::noneOp; actionOp = nullptr; }
    //
    inline void set_update(UpdateIndex_type update_index) { updateIndex = update_index; }
    void set_update(UpdateIndex_type update_index, const UpdatePA& update_);
    inline void unset_update() { updateIndex = ACTION::noneUpdate; update = nullptr; }

    [[nodiscard]] inline const AbstractState* _source() const { return source.get(); }
    [[nodiscard]] StateID_type _source_id() const;
    [[nodiscard]] inline const AbstractState* _target() const { return target.get(); }
    [[nodiscard]] StateID_type _target_id() const;

    [[nodiscard]] inline ActionLabel_type _action_label() const { return actionLabel; }
    [[nodiscard]] inline ActionOpID_type _action_op_id() const { return actionOpID; }
    [[nodiscard]] inline UpdateIndex_type _update_index() const { return updateIndex; }
    [[nodiscard]] inline const ActionOpPA* _action_op() const { return actionOp; }
    [[nodiscard]] inline const UpdatePA* _update() const { return update; }

    [[nodiscard]] inline bool has_action_label() const { return actionLabel != ACTION::noneLabel; }
    [[nodiscard]] inline bool has_action_op() const { return actionOpID != ACTION::noneOp; }
    [[nodiscard]] inline bool has_update() const { return updateIndex != ACTION::noneUpdate; }

};

#endif //PLAJA_PA_TRANSITION_STRUCTURE_H
