//
// This file is part of the PlaJA code base (2019 - 2021).
//

#ifndef PLAJA_INITIAL_STATES_ENUMERATOR_PA_H
#define PLAJA_INITIAL_STATES_ENUMERATOR_PA_H

#include <list>
#include <memory>
#include "../../../parser/ast/expression/forward_expression.h"
#include "../../../parser/using_parser.h"
#include "../../../utils/default_constructors.h"
#include "../inc_state_space/forward_inc_state_space_pa.h"
#include "../optimization/forward_optimization_pa.h"
#include "../pa_states/forward_pa_states.h"
#include "../smt/forward_smt_pa.h"
#include "../using_predicate_abstraction.h"

namespace PLAJA { class Configuration; }

class InitialStatesEnumeratorPa {

private:
    const SEARCH_SPACE_PA::IncStateSpace* stateSpace; // Information about predicate state order.
    const ModelZ3PA* modelZ3;
    std::unique_ptr<StateConditionExpression> stateCondition;
    std::unique_ptr<Z3_IN_PLAJA::SMTSolver> solver;
    std::unique_ptr<PredicateRelations> predicateRelations; // Own instance to include start condition as ground truth.

    /* Cache. */
    const std::size_t numLocations;
    std::size_t numPredicates;

    std::unique_ptr<StateValues> locationState;
    std::list<PredicateState> paStates;

    void enumerate(PredicateState& pred_state, PredicateTraverser& predicate_traverser);

public:
    explicit InitialStatesEnumeratorPa(const PLAJA::Configuration& config, const Expression* condition = nullptr);
    ~InitialStatesEnumeratorPa();
    DELETE_CONSTRUCTOR(InitialStatesEnumeratorPa)

    void increment(); // Incremental usage.
    void reset_smt();

    std::list<PredicateState> enumerate();
    std::list<PredicateState> enumerate(const AbstractState& base); // For incremental usage.

    [[nodiscard]] bool do_enumerate_explicitly() const { return not solver; }

    std::list<PredicateState> enumerate_explicitly();

};

#endif //PLAJA_INITIAL_STATES_ENUMERATOR_PA_H
