//
// This file is part of the PlaJA code base (2019 - 2021).
//

#include "action_op_pa.h"
#include "../../../successor_generation/action_op.h"
#include "../../../smt/solver/smt_solver_z3.h"
#include "../../smt/model_z3_pa.h"

ActionOpPA::ActionOpPA(const ActionOp& parent_, const ModelZ3PA& model_z3, Z3_IN_PLAJA::SMTSolver& solver):
    ActionOpAbstract(parent_, model_z3, false, solver.share_statistics()) {
    solver.push();
    add_src_bounds(solver);

    // updates (modified in derived class)
    compute_updates(solver);

    // cache operator encoding in z3:
    compute_op_in_z3();

    // finalize
    solver.pop();
}

ActionOpPA::~ActionOpPA() = default;

const ModelZ3PA& ActionOpPA::get_model_z3_pa() const { return static_cast<const ModelZ3PA&>(modelZ3); } // NOLINT(cppcoreguidelines-pro-type-static-cast-downcast)

void ActionOpPA::compute_updates(Z3_IN_PLAJA::SMTSolver& solver) {
    updates.reserve(actionOp.size());
    for (auto it = actionOp.updateIterator(); !it.end(); ++it) { updates.push_back(std::make_unique<UpdatePA>(it.update(), get_model_z3_pa(), solver)); }
}

void ActionOpPA::increment(Z3_IN_PLAJA::SMTSolver& solver) {
    solver.push();
    add_to_solver(solver);
    //
    for (auto& update: updates) { PLAJA_UTILS::cast_ref<UpdatePA>(*update).increment(solver); }
    //
    // check for incremental dynamic entailment computation:
    if (std::any_of(_guards_vars().cbegin(), _guards_vars().cend(), [this](VariableID_type var) { return get_model_z3_pa().get_vars_in_newly_added_predicates().count(var); })) {
        for (const auto& update: updates) { static_cast<UpdatePA&>(*update).lastAffectingPred = get_model_z3_pa().get_number_predicates(); } // NOLINT(cppcoreguidelines-pro-type-static-cast-downcast)
    } else {
        for (const auto& update: updates) {
            auto& update_pa = PLAJA_UTILS::cast_ref<UpdatePA>(*update);
            if (std::any_of(update_pa._updating_vars().cbegin(), update_pa._updating_vars().cend(), [this](VariableID_type var) { return get_model_z3_pa().get_vars_in_newly_added_predicates().count(var); })) {
                update_pa.lastAffectingPred = get_model_z3_pa().get_number_predicates();
            }
        }
    }
    //
    solver.pop();
}
