//
// This file is part of the PlaJA code base (2019 - 2021).
//

#ifndef PLAJA_SUCCESSOR_GENERATOR_PA_H
#define PLAJA_SUCCESSOR_GENERATOR_PA_H

#include "successor_generator_abstract.h"

// forward declaration:
class ActionOpPA;
class ModelZ3PA;


class SuccessorGeneratorPA: public SuccessorGeneratorAbstract {

private:
    void initialize_action_ops_pa();

    SuccessorGeneratorPA(const ModelZ3& model_z3, const PLAJA::Configuration& config);
public:
    ~SuccessorGeneratorPA() override;
    static std::unique_ptr<SuccessorGeneratorPA> construct(const ModelZ3PA& model_z3, const PLAJA::Configuration& config); // hack to allow splitting construction in derived classes

    DELETE_CONSTRUCTOR(SuccessorGeneratorPA)

    [[nodiscard]] const ModelZ3PA& get_model_z3_pa() const;

    void increment();

    [[nodiscard]] const ActionOpPA& get_action_op_pa(ActionOpID_type action_op_id) const;

    // iterators:

    using PerActionIterator = PerActionIteratorBase<ActionOpAbstract, ActionOpPA>;
    [[nodiscard]] inline PerActionIterator silentActionIteratorPA() const { return PerActionIterator(silentOps); }
    [[nodiscard]] inline PerActionIterator labeledActionIteratorPA(ActionLabel_type action_label) const { PLAJA_ASSERT(0 <= action_label && action_label <= labeledOps.size()) return PerActionIterator(labeledOps[action_label]); }

    using ActionIterator = ActionLabelIteratorBase<ActionOpAbstract, ActionOpPA>;
    [[nodiscard]] inline ActionIterator actionIteratorPA() const { return ActionIterator(silentOps, labeledOps); }

    using ActionOpIterator = ActionOpIteratorBase<ActionOpAbstract, ActionOpPA>;
    [[nodiscard]] inline ActionOpIterator actionOpIteratorPA() const { return ActionOpIterator(silentOps, labeledOps); }

};

/**********************************************************************************************************************/

// extern for interfacing:
namespace SUCCESSOR_GENERATOR_PA {
    [[nodiscard]] extern std::list<const ActionOpPA*> extract_ops_per_label(const SuccessorGeneratorPA& successor_generator, ActionLabel_type action_label);
};

#endif //PLAJA_SUCCESSOR_GENERATOR_PA_H
