//
// This file is part of the Marabou extension for PlaJA (2019 - 2023).
//

#ifndef PLAJA_SMT_SOLVER_H
#define PLAJA_SMT_SOLVER_H

#include <memory>
#include "../../../utils/default_constructors.h"
#include "../../../stats/forward_stats.h"
#include "forward_smt.h"

namespace PLAJA {

    /** Generic solver interface. */
    class SmtSolver {

    public:
        enum UnknownHandling { True, False, Error };

    protected:
        UnknownHandling unknownHandling; // How to handle undecided queries.
        bool unknownRlt;

        PLAJA::StatsBase* sharedStatistics; // Stats.

    public:
        explicit SmtSolver(PLAJA::StatsBase* stats);
        virtual ~SmtSolver() = 0;
        DELETE_CONSTRUCTOR(SmtSolver)

        inline void set(UnknownHandling unknown_handling) { unknownHandling = unknown_handling; }

        void add_smt_constraint(const PLAJA::SmtConstraint& constraint);

        virtual void push() = 0;

        virtual void pop() = 0;

        virtual void pop_repeated(unsigned int n);

        virtual void clear() = 0;

        /**
         * Fast check that may or may not derived sat.
         * Quick fix for large initialization overhead in Marabou solver.
         */
        virtual std::unique_ptr<bool> pre_check();

        virtual bool check() = 0;

    };

}

#endif //PLAJA_SMT_SOLVER_H
