//
// This file is part of PlaJA (2019 - 2024).
// Copyright (c) Marcel Vinzent (2024).
//

#ifndef PLAJA_CONSTRAINT_Z3_H
#define PLAJA_CONSTRAINT_Z3_H

#include <z3++.h>
#include "base/smt_constraint.h"
#include "forward_smt_z3.h"

namespace Z3_IN_PLAJA {

/**
 * Encapsulation of z3::expr expression.
 */
    class Constraint final: public PLAJA::SmtConstraint {

    private:
        z3::expr expr;

    public:
        explicit Constraint(z3::expr e):
            expr(std::move(e)) {
        }

        ~Constraint() override;
        DELETE_CONSTRUCTOR(Constraint)

        [[nodiscard]] inline const z3::expr& operator()() const { return expr; }

        void add_to_solver(PLAJA::SmtSolver& solver) const override;
        void add_negation_to_solver(PLAJA::SmtSolver& solver) const override;

    };

}

#endif //PLAJA_CONSTRAINT_Z3_H
