//
// This file is part of the PlaJA code base (2019 - 2023).
//

#include "output_constraints_in_z3.h"
#include <z3++.h>
#include "../solver/smt_solver_z3.h"
#include "../visitor/extern/to_z3_visitor.h"
#include "nn_in_z3.h"

Z3_IN_PLAJA::OutputConstraints::OutputConstraints(const NNInZ3& nn_in_z3) {

    const std::size_t num_outputs = nn_in_z3.get_output_layer_size();
    argmaxConstraints.reserve(num_outputs);

    for (auto output = 0; output < num_outputs; ++output) {

        const auto& output_var = nn_in_z3.get_output_var_z3(output);

        std::vector<z3::expr> argmax_constraints;
        argmax_constraints.reserve(num_outputs);

        OutputIndex_type other_output = 0;
        for (; other_output < output; ++other_output) {
            argmax_constraints.push_back(output_var > nn_in_z3.get_output_var_z3(other_output));
        }
        for (++other_output; other_output < num_outputs; ++other_output) {
            argmax_constraints.push_back(output_var >= nn_in_z3.get_output_var_z3(other_output));
        }

        argmaxConstraints.push_back(std::make_unique<z3::expr>(Z3_IN_PLAJA::to_conjunction(argmax_constraints)));
    }

}

Z3_IN_PLAJA::OutputConstraints::~OutputConstraints() = default;

//

void Z3_IN_PLAJA::OutputConstraints::add(Z3_IN_PLAJA::SMTSolver& solver, OutputIndex_type output) const { solver.add(get(output)); }