//
// This file is part of the PlaJA code base (2019 - 2020).
//

#ifndef PLAJA_SOLUTION_ITERATOR_BS_H
#define PLAJA_SOLUTION_ITERATOR_BS_H

#include <memory>
#include <list>
#include "../../../parser/using_parser.h"
#include "../../../utils/default_constructors.h"
#include "../using_smt.h"

// forward declaration:
class ModelZ3;
class StateValues;


/** Solution enumerator Binary Search */
class SolutionEnumeratorBS {
private:
    Z3_IN_PLAJA::SMTSolver* solver;
    const ModelZ3* modelZ3;

    std::list<StateValues> solutions;

    std::unique_ptr<Z3_IN_PLAJA::StateIndexes> stateIndexes; // variable (indexes) of interest

    // recursive enumeration routines:
    void enumerate_next(VariableIndex_type state_index, StateValues& state_values);
    void enumerate_bool(VariableIndex_type state_index, StateValues& state_values);
    void enumerate_integer(VariableIndex_type state_index, PLAJA::integer lower_bound, PLAJA::integer upper_bound, StateValues& state_values);
    void enumerate_floating(VariableIndex_type state_index, PLAJA::floating lower_bound, PLAJA::floating upper_bound, StateValues& state_values);

public:
    SolutionEnumeratorBS(Z3_IN_PLAJA::SMTSolver& solver_, const ModelZ3& model_z3);
    ~SolutionEnumeratorBS();
    DELETE_CONSTRUCTOR(SolutionEnumeratorBS)

    /**
     * Compute solutions given the ground truth of the solver (variable bounds must have been added).
     */
    static std::list<StateValues> compute_solutions(Z3_IN_PLAJA::SMTSolver& solver, const ModelZ3& model_z3, StateValues&& constructor);

};


#endif //PLAJA_SOLUTION_ITERATOR_BS_H
