//
// This file is part of the PlaJA code base (2019 - 2020).
//

#ifndef PLAJA_SOLUTION_ITERATOR_TD_H
#define PLAJA_SOLUTION_ITERATOR_TD_H

#include <list>
#include <memory>
#include "../../../utils/default_constructors.h"
#include "../using_smt.h"

// forward declaration:
class ModelZ3;
class StateValues;


/** Solution enumerator Tree Disjunction */
class SolutionEnumeratorTD { // TODO untested

private:
    Z3_IN_PLAJA::SMTSolver* solver;
    // cache
    std::unique_ptr<Z3_IN_PLAJA::StateIndexes> stateIndexes; // variable (indexes) of interest

    std::list<StateValues> solutions;

    // recursive enumeration routines:
    void enumerate(StateValues& constructor);

public:
    SolutionEnumeratorTD(Z3_IN_PLAJA::SMTSolver& solver_, const ModelZ3& model_z3);
    ~SolutionEnumeratorTD();
    DELETE_CONSTRUCTOR(SolutionEnumeratorTD)

    /**
     * Compute solutions given the ground truth of the solver (variable bounds must have been added).
     * @param solver
     * @param model_z3
     * @param constructor default state values, currently exactly the location values.
     * @return
     */
    static std::list<StateValues> compute_solutions(Z3_IN_PLAJA::SMTSolver& solver, const ModelZ3& model_z3, StateValues&& constructor);

};


#endif //PLAJA_SOLUTION_ITERATOR_TD_H
