//
// This file is part of the PlaJA code base (2019 - 2020).
//

#ifndef PLAJA_SOLUTION_ITERATOR_Z3_H
#define PLAJA_SOLUTION_ITERATOR_Z3_H

#include <list>
#include "../../../parser/using_parser.h"
#include "../../../utils/default_constructors.h"
#include "../using_smt.h"

// forward declaration:
class Model;
class ModelInformation;
class ModelZ3;
class StateValues;
class VariableDeclaration;

class SolutionEnumeratorZ3 {

private:
    Z3_IN_PLAJA::SMTSolver* solver;
    const ModelZ3* modelZ3;
    std::unique_ptr<Z3_IN_PLAJA::StateIndexes> stateIndexes; // variables of interest
    std::unique_ptr<Z3_IN_PLAJA::StateVarsInZ3> stateVars; // variables of interest

public:
    SolutionEnumeratorZ3(Z3_IN_PLAJA::SMTSolver& solver_, const ModelZ3& model_z3);
    ~SolutionEnumeratorZ3();
    DELETE_CONSTRUCTOR(SolutionEnumeratorZ3)

    void set_variables_of_interest(const std::vector<VariableID_type>& var_ids, bool use_target_vars);

    /**
     * @return true if a new solution was found, false otherwise.
     */
    bool get_solution(StateValues& solution);

    /**
     * Compute solutions given the ground truth of the solver (variable bounds must have been added).
     */
    static std::list<StateValues> compute_solutions(Z3_IN_PLAJA::SMTSolver& solver, const ModelZ3& model_z3, const StateValues& constructor);

};

#endif //PLAJA_SOLUTION_ITERATOR_Z3_H
