//
// This file is part of the PlaJA code base (2019 - 2023).
//

#ifndef PLAJA_SOLUTION_CHECK_WRAPPER_Z3_H
#define PLAJA_SOLUTION_CHECK_WRAPPER_Z3_H

#include <memory>
#include "../../../utils/default_constructors.h"
#include "../../../assertions.h"
#include "../../smt/base/solution_check_wrapper.h"
#include "../../states/forward_states.h"
#include "../forward_smt_z3.h"

namespace Z3_IN_PLAJA {

    class SolutionCheckWrapper: public ::SolutionCheckWrapper {

    private:
        const ModelZ3* model;
        const Z3_IN_PLAJA::Solution* solutionPtr; // resposibility of callee to keep object in memory

    protected:

        inline void set_solution_ptr(const Z3_IN_PLAJA::Solution& solution) { solutionPtr = &solution; }

    public:
        SolutionCheckWrapper(std::unique_ptr<::SolutionCheckerInstance>&& solution_checker_instance, const ModelZ3& model);
        ~SolutionCheckWrapper() override;
        DELETE_CONSTRUCTOR(SolutionCheckWrapper)

        [[nodiscard]] std::unique_ptr<StateValues> to_state(StepIndex_type step, bool include_locs) const override;

        void to_value(StepIndex_type step, StateValues& target, VariableIndex_type state_index) const override;

        /* z3 interface */

        [[nodiscard]] bool check(const Z3_IN_PLAJA::Solution& solution);

        void set(const Z3_IN_PLAJA::Solution& solution);

    };

}

#endif //PLAJA_SOLUTION_CHECK_WRAPPER_Z3_H
