//
// This file is part of the PlaJA code base (2019 - 2021).
//

#ifndef PLAJA_SUCCESSOR_GENERATOR_TO_Z_3_H
#define PLAJA_SUCCESSOR_GENERATOR_TO_Z_3_H

#include <list>
#include <memory>
#include "../../../utils/default_constructors.h"
#include "../../using_search.h"
#include "../using_smt.h"

// forward declaration
class ActionOpToZ3;
class ModelZ3;
class ModelInformation;
class SuccessorGeneratorC;

class SuccessorGeneratorToZ3 {

private:
    const ModelZ3& modelZ3;
    const ModelInformation* modelInformation;
    const SuccessorGeneratorC* successorGenerator;
    std::vector<std::unique_ptr<ActionOpToZ3>> nonSyncOps;
    std::vector<std::vector<std::unique_ptr<ActionOpToZ3>>> syncOps;
    //
    std::list<const ActionOpToZ3*> silentOps;
    std::vector<std::list<const ActionOpToZ3*>> labeledOps;

    // internal aux
    void compute_action_structure(ActionLabel_type action_label);

public:
    explicit SuccessorGeneratorToZ3(const ModelZ3& model_z3);
    ~SuccessorGeneratorToZ3();
    DELETE_CONSTRUCTOR(SuccessorGeneratorToZ3)

    [[nodiscard]] const ActionOpToZ3& get_action_op_structure(ActionOpID_type action_op_id) const;
    [[nodiscard]] const std::list<const ActionOpToZ3*>& get_action_structure(ActionLabel_type action_label) const;

    [[nodiscard]] z3::expr generate_action_op(ActionOpID_type action_op_id, const Z3_IN_PLAJA::StateVarsInZ3& src_vars, const Z3_IN_PLAJA::StateVarsInZ3& target_vars) const;
    [[nodiscard]] z3::expr generate_action(const std::list<const ActionOpToZ3*>& action_structure, const Z3_IN_PLAJA::StateVarsInZ3& src_vars, const Z3_IN_PLAJA::StateVarsInZ3& target_vars) const;
    [[nodiscard]] z3::expr generate_action(ActionLabel_type action_label, const Z3_IN_PLAJA::StateVarsInZ3& src_vars, const Z3_IN_PLAJA::StateVarsInZ3& target_vars) const;
    [[nodiscard]] z3::expr generate_expansion(const Z3_IN_PLAJA::StateVarsInZ3& src_vars, const Z3_IN_PLAJA::StateVarsInZ3& target_vars) const;

};

#endif //PLAJA_SUCCESSOR_GENERATOR_TO_Z_3_H
