//
// This file is part of the PlaJA code base (2019 - 2021).
//

#ifndef PLAJA_STATE_VALUES_H
#define PLAJA_STATE_VALUES_H

#include <memory>
#include "state_base.h"
#include "forward_states.h"

class StateValues final: public StateBase {

private:
    std::vector<PLAJA::integer> valuesInt;
    std::vector<PLAJA::floating> valuesFloating;

public:
    explicit StateValues(const State& state);
    explicit StateValues(std::vector<PLAJA::integer> values_int, std::vector<PLAJA::floating> values_float = {});
    explicit StateValues(std::size_t int_state_size, std::size_t floating_state_size = 0);
    ~StateValues() final = default;

    StateValues(const StateValues& other) = default;
    StateValues& operator=(const StateValues& other) = delete;
    StateValues(StateValues&& other) = default;
    StateValues& operator=(StateValues&& other) = default;

    /* Interface. */
    [[nodiscard]] std::size_t get_int_state_size() const override { return valuesInt.size(); }

    [[nodiscard]] std::size_t get_floating_state_size() const override { return valuesFloating.size(); }

    //
    [[nodiscard]] PLAJA::integer get_int(VariableIndex_type var) const override;
    [[nodiscard]] PLAJA::floating get_float(VariableIndex_type var) const override;
    //
    void set_int(VariableIndex_type var, PLAJA::integer value) override;
    void set_float(VariableIndex_type var, PLAJA::floating value) override;

    //

    [[nodiscard]] StateValues to_state_values() const override { return { *this }; }

    [[nodiscard]] inline std::unique_ptr<StateValues> to_ptr() const { return std::make_unique<StateValues>(*this); }

    FCT_IF_DEBUG([[nodiscard]] std::unique_ptr<const State> to_state() const;)

};

#endif //PLAJA_STATE_VALUES_H
