//
// This file is part of the PlaJA code base (2019 - 2020).
//

#ifndef PLAJA_TESTS_DEBUGGING_TEST_H
#define PLAJA_TESTS_DEBUGGING_TEST_H

#include "../../utils/test_header.h"
#include <memory>
#include "../../../parser/ast/expression/non_standard/predicate_abstraction_expression.h"
#include "../../../parser/ast/property.h"
#include "../../../search/fd_adaptions/state.h"
#include "../../../search/predicate_abstraction/heuristic/state_queue.h"
#include "../../../search/predicate_abstraction/nn/nn_sat_checker/nn_sat_checker.h"
#include "../../../search/predicate_abstraction/pa_states/predicate_state.h"
#include "../../../search/predicate_abstraction/predicate_abstraction.h"
#include "../../../search/predicate_abstraction/search_space/search_space_pa.h"
#include "../../../search/predicate_abstraction/smt/model_z3_pa.h"
#include "../../../search/predicate_abstraction/successor_generation/action_op/action_op_pa.h"
#include "../../../search/predicate_abstraction/successor_generation/successor_generator_pa.h"
#include "../../../search/states/state_values.h"
#include "../../utils/test_header.h"

#include "../../../search/predicate_abstraction/base/search_pa_macros.h"

const std::string filename("../../../../tests/test_instances/for_ppa_racetrack.jani"); // NOLINT(*-err58-cpp)

class DebuggingTest: public PLAJA_TEST::TestSuit {

private:
    // structures
    PLAJA::ModelConstructorForTests modelConstructor;
    std::unique_ptr<PredicateAbstraction> ppa;
    //
    std::unique_ptr<State> source_state;

public:

    DebuggingTest() {
        // parse test Model
        modelConstructor.set_value_option("nn_for_racetrack", "car0_x,car0_y");
        modelConstructor.set_value_option("additional-properties", "../../../../tests/test_instances/tiny_c_car_v_car_relaxed.jani");
        modelConstructor.set_value_option("nn-sat", "z3");
        modelConstructor.construct_model(filename);

        ppa.reset(dynamic_cast<PredicateAbstraction*>(modelConstructor.construct_instance(SearchEngineFactory::PA_ENGINE, 8).release()));
        ppa->initialize();
        ppa->frontier.clear();
        // reset initial state to be unreached
        // ppa->searchSpace->get_node(ppa->searchSpace->_initialState()).set_unreached();

        // construct new source state:
        auto abstract_state = ppa->searchSpace->get_abstract_initial_state();
        auto location_state = abstract_state.to_location_state();
        auto predicate_state = abstract_state.to_predicate_state();
        // { 2, 18, 1, 0, 0, 0, 0, 1 }
        location_state.assign(0, 2);
        location_state.assign(1, 18);
        predicate_state.set(0, true);
        predicate_state.set(1, false);
        predicate_state.set(2, false);
        predicate_state.set(3, false);
        predicate_state.set(4, false);
        predicate_state.set(5, true);
        auto abstract_source_state = ppa->searchSpace->set_abstract_state(location_state, predicate_state);
        // set new source state:
        ppa->frontier->push(*abstract_source_state);
        SearchNodePA& node = ppa->searchSpace->add_node(*abstract_source_state);
        node.set_reached_init(abstract_source_state->get_id_value());

        // State::suppress_writtenTo(true); // as tests modify state objects
    }

    ~DebuggingTest() override = default;

/**********************************************************************************************************************/

    void testPA() {
        std::cout << std::endl << "Test NN-Sat check:" << std::endl;

        ppa->step();
    }

/**********************************************************************************************************************/

    void run_tests() override { RUN_TEST(testPA()) }

};

TEST_MAIN(DebuggingTest)

#endif //PLAJA_TESTS_DEBUGGING_TEST_H
